﻿namespace HeuristicLab.Algorithms.PushGP.Expressions {
  using HeuristicLab.Algorithms.PushGP.Generators;
  using HeuristicLab.Algorithms.PushGP.Interpreter;

  /// <summary>
  /// ERC expressions are only used for code generation.
  /// </summary>
  /// <typeparam name="T"></typeparam>
  public abstract class ErcExpression<T> : PushExpression<T> {
    protected ErcExpression(T value) : base(value) { }

    protected override string InitStringRepresentation() {
      return string.Format("{0}({1})", this.GetType().Name, this.State);
    }

    protected override int InitId() {
      return this.State.GetHashCode();
    }
  }

  public class IntegerErcExpression : ErcExpression<long> {
    public IntegerErcExpression() : base(IntegerGenerator.RandomInteger()) { }

    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter.IntegerStack);
    }
  }

  public class FloatErcExpression : ErcExpression<double> {
    public FloatErcExpression() : base(FloatGenerator.RandomFloat()) { }

    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter.FloatStack);
    }
  }

  public class BooleanErcExpression : ErcExpression<bool> {
    public BooleanErcExpression() : base(BooleanGenerator.RandomBoolean()) { }

    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter.BooleanStack);
    }
  }

  public class NameErcExpression : ErcExpression<string> {
    public NameErcExpression() : base(NameGenerator.RandomName()) { }

    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter.NameStack);
    }
  }
}